/*
 * utds.c
 *	Support 9270 VLBI data-sampler (USB device)
 *	written by KIYOHARA Takashi <kiyohara@soum.co.jp>
 *	$SOUM$
 */
#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/init.h>
#include <linux/module.h>
#include <linux/slab.h>

#include <asm/semaphore.h>
#include <asm/uaccess.h>

#include <linux/usb.h>


/* XXXX: tricky definitions */
#define UTDS_DO_GARBAGE_CLEAR
#define UTDS_DATAIN_512_ONLY
#define UTDS_COMMAND_DMA_RETRIES	3
#define SKIP_HEADER_ERROR

#include <tdsio.h>
#include <tdssdh.h>


#ifdef TDS_DEBUG
int utds_debug = TDS_DEBUG;
#define DPRINTF(fmt, args...)		printk(KERN_DEBUG fmt, ## args);
#define DPRINTFN(level, fmt, args...)				\
	do {							\
		if ((level) <= utds_debug)			\
			printk(KERN_DEBUG fmt, ## args);	\
	} while (0)

#define IFDBG(cond)			if((cond))
#define IFDBGN(level, cond)		if((level) <= utds_debug && (cond))

#define UTDS_INLINE	__inline
#else
#define DPRINTF(fmt, args...)		/* none */
#define DPRINTFN(level, fmt, args...)	/* none */

#define IFDBG(cond)			if(0)
#define IFDBGN(level, cond)		if(0)

#define UTDS_INLINE			/* none */
#endif


#define USB_VENDOR_ID_CYPRESS		0x04b4
#define USB_DEVICE_ID_NITSUKI_9270VLBI	0x9270

#define USB_UTDS_MINOR			222		/* XXXX */

#define UTDS_IOBUF_SIZE			(13 * 512 * 2)	/* Max Bulk x2 */
#define UTDS_IOBUF_NUM			4096

#define UTDS_EP_DATAIN			2
#define UTDS_EP_CMDOUT			6
#define UTDS_EP_CMDIN			8

#define UTDS_SYNC_PATTERN		0x8c

#define IS_SYNC_PATTERN(hdr)			\
	((hdr)->syncptn0 == 0xffff &&		\
	    (hdr)->syncptn1 == 0xffff &&	\
	    (hdr)->syncptn2 == UTDS_SYNC_PATTERN)

/* 9270 VLBI Time Sync Data Sampler Registers */
#define UTDSREG_RREG_BEGIN	0x00
#define UTDSREG_WREG_BEGIN	0x10
#define UTDSREG_END		0x1f
#define UTDSREG_WRITE_BIT	0x80

/* Read-Only */
#define UTDSREG_STAT		0x00
#define UTDSREG_DDRMADDRW0	0x01
#define UTDSREG_DDRMADDRW1	0x02
#define UTDSREG_DDRMADDRW2	0x03
#define UTDSREG_DDRMADDRW3	0x04
#define UTDSREG_DDRMADDRR0	0x05
#define UTDSREG_DDRMADDRR1	0x06
#define UTDSREG_DDRMADDRR2	0x07
#define UTDSREG_DDRMADDRR3	0x08
#define UTDSREG_SPACE		0x0f		/* XXXXX */

/* Read/Write */
#define UTDSREG_SAMPLING	0x10
#define UTDSREG_FILTER		0x10
#define UTDSREG_RESOLUTIONBIT	0x10
#define UTDSREG_CHMODE		0x10
#define UTDSREG_FSAMPLE		0x11
#define UTDSREG_START		0x12
#define UTDSREG_COMMAND		0x13
#define UTDSREG_TIME0		0x14
#define UTDSREG_TIME1		0x15
#define UTDSREG_TIME2		0x16
#define UTDSREG_TIME3		0x17
#define UTDSREG_YEAR		0x18
#define UTDSREG_DCOFFSET_CH1	0x1c
#define UTDSREG_DCOFFSET_CH2	0x1d
#define UTDSREG_DCOFFSET_CH3	0x1e
#define UTDSREG_DCOFFSET_CH4	0x1f

/* 9270 VLBI Time Sync Data Sampler Bits */
#define UTDSREG_STAT_REFMASK		0x20
#define UTDSREG_STAT_10MHZ		0x00
#define UTDSREG_STAT_5MHZ		0x20
#define UTDSREG_STAT_PLLLOCK		0x10
#define UTDSREG_STAT_REFINPUT		0x08
#define UTDSREG_STAT_1PPSINPUT		0x04
#define UTDSREG_STAT_DATAEXIST		0x02
#define UTDSREG_STAT_ONSAMPLING		0x01

/*	2006/11/24	H.Ohkubo	*/
#define UTDSREG_STAT_FIFOOVFLW		0x80

#define UTDSREG_DDRMADDR(v0, v1, v2, v3) \
    (((v0) | (v1) << 8 | (v2) << 16 || (v3) << 24) & UTDSREG_DDRMADDR_MASK)
#define UTDSREG_DDRMADDR_MASK		0x01ffffff

#define GET_UTDSREG_FILTER(v) \
    (((v) & UTDSREG_FILTER_MASK) << UTDSREG_FILTER_SHIFT)
#define GET_UTDSIO_FILTER(r) \
    (((r) >> UTDSREG_FILTER_SHIFT) & UTDSREG_FILTER_MASK)
#define UTDSREG_FILTER_SHIFT		3
#define UTDSREG_FILTER_MASK		0x07
#define UTDSREG_FILTER_2M		0x04
#define UTDSREG_FILTER_4M		0x03
#define UTDSREG_FILTER_8M		0x02
#define UTDSREG_FILTER_16M		0x01
#define UTDSREG_FILTER_THRU		0x00

#define GET_UTDSREG_RESOLUTIONBIT(v) \
    (((v) & UTDSREG_RESOLUTIONBIT_MASK) << UTDSREG_RESOLUTIONBIT_SHIFT)
#define GET_UTDSIO_RESOLUTIONBIT(r) \
    (((r) >> UTDSREG_RESOLUTIONBIT_SHIFT) & UTDSREG_RESOLUTIONBIT_MASK)
#define UTDSREG_RESOLUTIONBIT_SHIFT	1
#define UTDSREG_RESOLUTIONBIT_MASK	0x03
#define UTDSREG_RESOLUTIONBIT_8BIT	0x03
#define UTDSREG_RESOLUTIONBIT_4BIT	0x02
#define UTDSREG_RESOLUTIONBIT_2BIT	0x01
#define UTDSREG_RESOLUTIONBIT_1BIT	0x00

#define GET_UTDSREG_CHMODE(v) \
    (((v) & UTDSREG_CHMODE_MASK) << UTDSREG_CHMODE_SHIFT)
#define GET_UTDSIO_CHMODE(r) \
    (((r) >> UTDSREG_CHMODE_SHIFT) & UTDSREG_CHMODE_MASK)
#define UTDSREG_CHMODE_SHIFT		0
#define UTDSREG_CHMODE_MASK		0x01
#define UTDSREG_CHMODE_1CH		0x00
#define UTDSREG_CHMODE_4CH		0x01

#define GET_UTDSREG_FSAMPLE(v) \
    (((v) & UTDSREG_FSAMPLE_MASK) << UTDSREG_FSAMPLE_SHIFT)
#define GET_UTDSIO_FSAMPLE(r) \
    (((r) >> UTDSREG_FSAMPLE_SHIFT) & UTDSREG_FSAMPLE_MASK)
#define UTDSREG_FSAMPLE_SHIFT		0
#define UTDSREG_FSAMPLE_MASK		0x0f
#define UTDSREG_FSAMPLE_64M		0x0a
#define UTDSREG_FSAMPLE_32M		0x09
#define UTDSREG_FSAMPLE_16M		0x08
#define UTDSREG_FSAMPLE_8M		0x07
#define UTDSREG_FSAMPLE_4M		0x06
#define UTDSREG_FSAMPLE_2M		0x05
#define UTDSREG_FSAMPLE_1M		0x04
#define UTDSREG_FSAMPLE_05M		0x03
#define UTDSREG_FSAMPLE_02M		0x02
#define UTDSREG_FSAMPLE_01M		0x01
#define UTDSREG_FSAMPLE_004M		0x00

#define UTDSREG_START_SAMPLESTRT	0x01

#define UTDSREG_COMMAND_RESET		0x04
#define UTDSREG_COMMAND_BUFFCLEAR	0x02
#define UTDSREG_COMMAND_1PPSSYNC	0x01

#define UTDS_FILTER_DEFAULT		UTDSREG_FILTER_THRU
#define UTDS_RESOLUTIONBIT_DEFAULT	UTDSREG_RESOLUTIONBIT_1BIT
#define UTDS_CHMODE_DEFAULT		UTDSREG_CHMODE_1CH
#define UTDS_FSAMPLE_DEFAULT		UTDSREG_FSAMPLE_004M
#define UTDS_AUXSIZE_DEFAULT	20


static struct usb_device_id utds_table[] = {
	{ USB_DEVICE(USB_VENDOR_ID_CYPRESS, USB_DEVICE_ID_NITSUKI_9270VLBI) },

	{ 0, 0 }				/* Terminating entry */
};

MODULE_DEVICE_TABLE(usb, utds_table);
 
MODULE_AUTHOR("NIHON TSUSHINKI CO, LTD.");
MODULE_DESCRIPTION("Driver for Nitsuki Time Sync Data Sampler 9270");
MODULE_LICENSE("");


static uint32_t utds_ver = 0x00010000;

char utds_id[] = "$SOUM$";
static struct utds_core {
	const char name[16];
	char *id;
} utds_core = {
	name:	"utds",
	id:	utds_id,
};
static struct utds_core *core = &utds_core;
/* prevent races between open() and disconnect() */
static DECLARE_MUTEX(discsem);

enum { datain = 0, cmdout = 1, cmdin = 2, maxendpoints = 3 };
struct utds {
	struct usb_device *udev;
	struct usb_interface *iface;

	int dv_unit;			/* unit number */
	char dv_xname[16];		/* device name and uniit number */

        struct utdsep {
		int addr;		/* endpoint address */
		int dir;		/* direction */
		int xfertype;		/* attribute xfer type */
		int pktsize;		/* packet size */

		int pipe;		/* sendpipe */

		struct utds *dev;	/* back pointer to the utds */
		struct urb *urb;
		void *dmabuf;
		dma_addr_t dmaaddr;
		size_t bufsize;
		atomic_t busy;		/* urb busy flag */
		struct completion xdone;
	} endpoints[maxendpoints];

	int dma_index;			/* DMA index for buffer */
	int io_index;			/* I/O index for buffer */
	off_t io_offset;		/* I/O offset for buffer */
	struct utds_iobuf {
		void *buffer;
		dma_addr_t dmaaddr;
	} iobuf[UTDS_IOBUF_NUM];
#ifdef UTDS_DO_GARBAGE_CLEAR
	int initread;
	int initcheck;
	int refound;
#endif

	struct tdsaux auxdata;		/* AUX data buffer */
	int totalsize;			/* total size (header + data) */
	int pktoffset;

	int sampling_filter;
	int sampling_resolutionbit;
	int sampling_chmode;
	int sampling_fsample;

	int error;
	unsigned long flags;
#define UTDS_FLAGS_SAMPLING_START	0
#define UTDS_FLAGS_SAMPLING_DONE	1
#define UTDS_FLAGS_PPS_SYNC_DONE	2
#define UTDS_FLAGS_TIME_SET_DONE	3

	int present;			/* if the device is not disconnected */
	int open;			/* if the port is open or not */
	struct semaphore iosem;

	wait_queue_head_t waitq;
};
typedef struct utds *utds_t;
typedef struct utdsep *utdsep_t;

struct lookuptbl {
	int key;
	int value;
};
static struct lookuptbl fsampletbl[] = {
	{	       (40 * 1000),	TDSIO_SAMPLING_40KHZ	},
	{	      (100 * 1000),	TDSIO_SAMPLING_100KHZ	},
	{	      (200 * 1000),	TDSIO_SAMPLING_200KHZ	},
	{	      (500 * 1000),	TDSIO_SAMPLING_500KHZ	},
	{	 (1 * 1000 * 1000),	TDSIO_SAMPLING_1MHZ	},
	{	 (2 * 1000 * 1000),	TDSIO_SAMPLING_2MHZ	},
	{	 (4 * 1000 * 1000),	TDSIO_SAMPLING_4MHZ	},
	{	 (8 * 1000 * 1000),	TDSIO_SAMPLING_8MHZ	},
	{	(16 * 1000 * 1000),	TDSIO_SAMPLING_16MHZ	},
	{	(32 * 1000 * 1000),	TDSIO_SAMPLING_32MHZ	},
	{	(64 * 1000 * 1000),	TDSIO_SAMPLING_64MHZ	},
	{	0,			-1 			}
};
static struct lookuptbl resolutionbittbl[] = {
	{	1,	TDSIO_SAMPLING_1BIT	},
	{	2,	TDSIO_SAMPLING_2BIT	},
	{	4,	TDSIO_SAMPLING_4BIT	},
	{	8,	TDSIO_SAMPLING_8BIT	},
	{	0,	-1	 		}
};
static struct lookuptbl chmodetbl[] = {
	{	1,	TDSIO_SAMPLING_1CH	},
	{	4,	TDSIO_SAMPLING_4CH	},
	{	0,	-1	 		}
};
static struct lookuptbl filtertbl[] = {
	{	2,	TDSIO_SAMPLING_2M	},
	{	4,	TDSIO_SAMPLING_4M	},
	{	8,	TDSIO_SAMPLING_8M	},
	{	16,	TDSIO_SAMPLING_16M	},
	{	0,	TDSIO_SAMPLING_THRU	},
	{	0,	-1	 		}
};


static int inline lookup_value(struct lookuptbl *, int);
static UTDS_INLINE void utds_garbage_clear_command(utds_t dev);
static UTDS_INLINE int utdsreg_get_status(utds_t);
static UTDS_INLINE void utdsreg_set_sampling(utds_t, int, int, int, int);
static UTDS_INLINE void utdsreg_start_sampling(utds_t);
static UTDS_INLINE void utdsreg_stop_sampling(utds_t);
static UTDS_INLINE void utdsreg_reset_memory(utds_t);
static UTDS_INLINE void utdsreg_clear_memory(utds_t);
static UTDS_INLINE void utdsreg_sync_1pps(utds_t);
static UTDS_INLINE void utdsreg_set_time(utds_t, uint32_t);
static UTDS_INLINE uint32_t utdsreg_get_time(utds_t);
static UTDS_INLINE void utdsreg_set_year(utds_t, uint32_t);
static UTDS_INLINE uint32_t utdsreg_get_year(utds_t);
static UTDS_INLINE int utdsreg_get_dcoffset(utds_t, int);
static UTDS_INLINE int utdsreg_set_dcoffset(utds_t, int, int);

static int utds_get_config(int, utdsep_t);
static int utds_command_dma(utdsep_t, int);
static void utds_command_completion(struct urb *, struct pt_regs *);
static void utds_command_timeout(unsigned long);
static int utds_data_dma(utdsep_t);
static void utds_data_completion(struct urb *, struct pt_regs *);

static UTDS_INLINE void utds_init(utds_t);
static UTDS_INLINE int utds_calc_bytes(utds_t);
static UTDS_INLINE int utdsio_sampling_start(utds_t);
static UTDS_INLINE int utdsio_sampling_stop(utds_t);
static UTDS_INLINE void utds_dma_buffer_init(utds_t);
static UTDS_INLINE void utds_issue_dma(utds_t);       

static ssize_t utds_read(struct file *, char *, size_t, loff_t *);
static int
    utds_ioctl(struct inode *, struct file *, unsigned int, unsigned long);
static int utds_open(struct inode *, struct file *);
static int utds_release(struct inode *, struct file *);

static void utds_print_info(utds_t);
static int utds_probe(struct usb_interface *, const struct usb_device_id *);
static void utds_disconnect(struct usb_interface *);


static struct file_operations utds_fops = {
	.owner =        THIS_MODULE,

	.read =		utds_read,
	.ioctl =	utds_ioctl,
	.open =		utds_open,
	.release =	utds_release,
};

static struct usb_class_driver utds_class = {
	.name =		"usb/utds%d",
	.fops =		&utds_fops,
	.mode =		S_IFCHR | S_IRUSR | S_IWUSR |
			    S_IRGRP | S_IWGRP | S_IROTH,
	.minor_base =	USB_UTDS_MINOR,
};

static struct usb_driver utds_driver = {
	.owner =	THIS_MODULE,
	.name =		"utds",
	.probe =	utds_probe,
	.disconnect =	utds_disconnect,
	.id_table =	utds_table, 
};


static int inline
lookup_value(struct lookuptbl *tbl, int value)
{
	int i;

	for (i = 0; (tbl + i)->value != -1; i++)
		if ((tbl + i)->value == value)
			return (0);
	return (-EINVAL);
}


#ifdef UTDS_DO_GARBAGE_CLEAR
static UTDS_INLINE void
utds_garbage_clear_command(utds_t dev)
{
	dma_addr_t dma;
	int rv = 0;
	char *data;

	if ((data = usb_buffer_alloc(dev->udev, PAGE_SIZE, GFP_KERNEL, &dma)) ==
	    NULL) {
		dev->error = TDS_ERROR_NO_DMA_BUFFER;
		rv = -ENOMEM;
		err("%s: usb_buffer_alloc failed %d", dev->dv_xname, rv);
	} else
		if ((rv = usb_get_descriptor(
		    dev->udev, USB_DT_CONFIG, 0, data, PAGE_SIZE)) < 0) {
			dev->error = TDS_ERROR_NO_DMA_BUFFER;
			err("%s: utds_garbage_clear_command failed %d",
			    dev->dv_xname, rv);
		}
}
#endif


/* register access routines for DataSampler */

static UTDS_INLINE int
utdsreg_get_status(utds_t dev)
{
	utdsep_t epin, epout;
	int rv, cnt = 0;
	unsigned char *buf;

	epout = &dev->endpoints[cmdout];
	epin = &dev->endpoints[cmdin];

	buf = epout->dmabuf;
	buf[0] = UTDSREG_STAT;
	buf[1] = 0;
	if ((rv = utds_command_dma(epout, 2)) < 0)
		return (rv);

retry:
	if ((rv = utds_command_dma(epin, 2)) < 0)
		return (rv);
	buf = epin->dmabuf;
	if (buf[0] != UTDSREG_STAT) {
		err("%s: %s: recieved command address failed: %02x:%02x\n",
		    dev->dv_xname, __FUNCTION__, buf[0], buf[1]);
		if (cnt ++ < UTDS_COMMAND_DMA_RETRIES) {
			err("retry %d\n", cnt);
			goto retry;
		}
		dev->error = TDS_ERROR_IO_ERROR;
		return (-EIO);
	}
	return (buf[1]);
}

static UTDS_INLINE void
utdsreg_set_sampling(utds_t dev,
    int filter, int resolutionbit, int chmode, int fsample)
{
	utdsep_t endpoint = &dev->endpoints[cmdout];
	unsigned char *buf = endpoint->dmabuf;

	buf[0] = UTDSREG_WRITE_BIT | UTDSREG_SAMPLING;
	buf[1] =
	    GET_UTDSREG_FILTER(filter) |
	    GET_UTDSREG_RESOLUTIONBIT(resolutionbit) |
	    GET_UTDSREG_CHMODE(chmode);
	if (utds_command_dma(endpoint, 2) < 0)
		printk("utdsreg_set_sampling: failed\n");
	buf[0] = UTDSREG_WRITE_BIT | UTDSREG_FSAMPLE;
	buf[1] = GET_UTDSREG_FSAMPLE(fsample);
	if (utds_command_dma(endpoint, 2) < 0)
		printk("utdsreg_set_sampling: failed\n");
}

static UTDS_INLINE void
utdsreg_start_sampling(utds_t dev)
{
	utdsep_t endpoint = &dev->endpoints[cmdout];
	unsigned char *buf = endpoint->dmabuf;

	buf[0] = UTDSREG_WRITE_BIT | UTDSREG_START;
	buf[1] = UTDSREG_START_SAMPLESTRT;
	if (utds_command_dma(endpoint, 2) < 0)
		printk("utdsreg_start_sampling: failed\n");
}

static UTDS_INLINE void
utdsreg_stop_sampling(utds_t dev)
{
	utdsep_t endpoint = &dev->endpoints[cmdout];
	unsigned char *buf = endpoint->dmabuf;

	buf[0] = UTDSREG_WRITE_BIT | UTDSREG_START;
	buf[1] = 0;
	if (utds_command_dma(endpoint, 2) < 0)
		printk("utdsreg_stop_sampling: failed\n");
}

static UTDS_INLINE void
utdsreg_reset_memory(utds_t dev)
{
	utdsep_t endpoint = &dev->endpoints[cmdout];
	unsigned char *buf = endpoint->dmabuf;

	buf[0] = UTDSREG_WRITE_BIT | UTDSREG_COMMAND;
	buf[1] = UTDSREG_COMMAND_BUFFCLEAR | UTDSREG_COMMAND_RESET; /* set */
	if (utds_command_dma(endpoint, 2) < 0)
		printk("utdsreg_reset_memory: failed\n");
	buf[0] = UTDSREG_WRITE_BIT | UTDSREG_COMMAND;
	buf[1] = 0;						/* reset ? */
	if (utds_command_dma(endpoint, 2) < 0)
		printk("utdsreg_reset_memory: failed\n");
}

static UTDS_INLINE void
utdsreg_clear_memory(utds_t dev)
{
	utdsep_t endpoint = &dev->endpoints[cmdout];
	unsigned char *buf = endpoint->dmabuf;

	buf[0] = UTDSREG_WRITE_BIT | UTDSREG_COMMAND;
	buf[1] = UTDSREG_COMMAND_BUFFCLEAR;			/* set */
	if (utds_command_dma(endpoint, 2) < 0)
		printk("utdsreg_clear_memory: failed\n");
	buf[0] = UTDSREG_WRITE_BIT | UTDSREG_COMMAND;
	buf[1] = 0;						/* reset ? */
	if (utds_command_dma(endpoint, 2) < 0)
		printk("utdsreg_clear_memory: failed\n");
}

static UTDS_INLINE void
utdsreg_sync_1pps(utds_t dev)
{
	utdsep_t endpoint = &dev->endpoints[cmdout];
	unsigned char *buf = endpoint->dmabuf;

	buf[0] = UTDSREG_WRITE_BIT | UTDSREG_COMMAND;
	buf[1] = UTDSREG_COMMAND_1PPSSYNC;			/* set */
	if (utds_command_dma(endpoint, 2) < 0)
		printk("utdsreg_sync_1pps: failed\n");
	buf[0] = UTDSREG_WRITE_BIT | UTDSREG_COMMAND;
	buf[1] = 0;						/* reset ? */
	if (utds_command_dma(endpoint, 2) < 0)
		printk("utdsreg_sync_1pps: failed\n");
}

static UTDS_INLINE void
utdsreg_set_time(utds_t dev, uint32_t time)
{
	utdsep_t endpoint = &dev->endpoints[cmdout];
	unsigned char *buf = endpoint->dmabuf;

	time &= (TDS_SEC_MASK | TDS_DAYS_MASK);

	buf[0] = UTDSREG_WRITE_BIT | UTDSREG_TIME0;
	buf[1] = time;
	if (utds_command_dma(endpoint, 2) < 0)
		printk("utdsreg_set_time: failed\n");
	buf[0] = UTDSREG_WRITE_BIT | UTDSREG_TIME1;
	buf[1] = time >> 8;
	if (utds_command_dma(endpoint, 2) < 0)
		printk("utdsreg_set_time: failed\n");
	buf[0] = UTDSREG_WRITE_BIT | UTDSREG_TIME2;
	buf[1] = time >> 16;
	if (utds_command_dma(endpoint, 2) < 0)
		printk("utdsreg_set_time: failed\n");
	buf[0] = UTDSREG_WRITE_BIT | UTDSREG_TIME3;
	buf[1] = time >> 24;
	if (utds_command_dma(endpoint, 2) < 0)
		printk("utdsreg_set_time: failed\n");
}

static UTDS_INLINE uint32_t
utdsreg_get_time(utds_t dev)
{
	utdsep_t epin, epout;
	int rv;
	unsigned char *buf;
	int reg, i, cnt = 0;
	uint32_t value = 0;

	epout = &dev->endpoints[cmdout];
	epin = &dev->endpoints[cmdin];

	for (reg = UTDSREG_TIME0, i = 0; reg <= UTDSREG_TIME3; reg++, i++) {
		buf = epout->dmabuf;
		buf[0] = reg;
		buf[1] = 0;
		if ((rv = utds_command_dma(epout, 2)) < 0)
			return (rv);

retry:
		if ((rv = utds_command_dma(epin, 2)) < 0)
			return (rv);
		buf = epin->dmabuf;
		if (buf[0] != reg) {
			err("%s: %s: "
			    "recieved command address failed: %02x:%02x\n",
			    dev->dv_xname, __FUNCTION__, buf[0], buf[1]);
			if (cnt ++ < UTDS_COMMAND_DMA_RETRIES) {
				err("retry %d\n", cnt);
				goto retry;
			}
			dev->error = TDS_ERROR_IO_ERROR;
			return (-EIO);
		}
		value |= (buf[1] << (i * 8));
	}

	return (value);
}

static UTDS_INLINE void
utdsreg_set_year(utds_t dev, uint32_t year)
{
	utdsep_t endpoint = &dev->endpoints[cmdout];
	unsigned char *buf = endpoint->dmabuf;

	buf[0] = UTDSREG_WRITE_BIT | UTDSREG_YEAR;
	buf[1] = year;
	if (utds_command_dma(endpoint, 2) < 0)
		printk("utdsreg_set_seconds: failed\n");
}

static UTDS_INLINE uint32_t
utdsreg_get_year(utds_t dev)
{
	utdsep_t epin, epout;
	int rv, cnt = 0;
	unsigned char *buf;

	epout = &dev->endpoints[cmdout];
	epin = &dev->endpoints[cmdin];

	buf = epout->dmabuf;
	buf[0] = UTDSREG_YEAR;
	buf[1] = 0;
	if ((rv = utds_command_dma(epout, 2)) < 0)
		return (rv);

retry:
	if ((rv = utds_command_dma(epin, 2)) < 0)
		return (rv);
	buf = epin->dmabuf;
	if (buf[0] != UTDSREG_YEAR) {
		err("%s: %s: recieved command address failed: %02x:%02x\n",
		    dev->dv_xname, __FUNCTION__, buf[0], buf[1]);
		if (cnt ++ < UTDS_COMMAND_DMA_RETRIES) {
			err("retry %d\n", cnt);
			goto retry;
		}
		dev->error = TDS_ERROR_IO_ERROR;
		return (-EIO);
	}
	return (buf[1]);
}

static UTDS_INLINE int
utdsreg_set_dcoffset(utds_t dev, int ch, int val)
{
#define UTDS_CH2DCOFFSETREG(ch)		(ch + UTDSREG_DCOFFSET_CH1)

	utdsep_t endpoint = &dev->endpoints[cmdout];
	int rv;
	uint8_t reg = UTDS_CH2DCOFFSETREG(ch);
	unsigned char *buf = endpoint->dmabuf;

	if (reg < UTDSREG_DCOFFSET_CH1 || reg > UTDSREG_DCOFFSET_CH4) {
		dev->error = TDS_ERROR_IO_ERROR;
		return (-EINVAL);
	}

	buf[0] = UTDSREG_WRITE_BIT | reg;
	buf[1] = val;
	if ((rv = utds_command_dma(endpoint, 2)) < 0)
		printk("utdsreg_set_dcoffset: failed\n");

	return (rv);
}

static UTDS_INLINE int
utdsreg_get_dcoffset(utds_t dev, int ch)
{
	utdsep_t epin, epout;
	int rv, cnt = 0;
	uint8_t reg = UTDS_CH2DCOFFSETREG(ch);
	unsigned char *buf;

	if (reg < UTDSREG_DCOFFSET_CH1 || reg > UTDSREG_DCOFFSET_CH4) {
		dev->error = TDS_ERROR_IO_ERROR;
		return (-EINVAL);
	}

	epout = &dev->endpoints[cmdout];
	epin = &dev->endpoints[cmdin];

	buf = epout->dmabuf;
	buf[0] = reg;
	buf[1] = 0;
	if ((rv = utds_command_dma(epout, 2)) < 0)
		return (rv);

retry:
	if ((rv = utds_command_dma(epin, 2)) < 0)
		return (rv);
	buf = epin->dmabuf;
	if (buf[0] != reg) {
		err("%s: %s: recieved command address failed: %02x:%02x\n",
		    dev->dv_xname, __FUNCTION__, buf[0], buf[1]);
		if (cnt ++ < UTDS_COMMAND_DMA_RETRIES) {
			err("retry %d\n", cnt);
			goto retry;
		}
		dev->error = TDS_ERROR_IO_ERROR;
		return (-EIO);
	}
	return (buf[1]);
}


/* USB transfer routines */

static int
utds_get_config(int addr, utdsep_t endpoint)
{
	struct usb_endpoint_descriptor *epdesc = NULL;
	struct usb_descriptor_header *desc;
	utds_t dev = endpoint->dev;
	dma_addr_t dma;
	int found, i, rv = 0;
	char *data;

	if ((data = usb_buffer_alloc(dev->udev, PAGE_SIZE, GFP_KERNEL, &dma)) ==
	    NULL) {
		dev->error = TDS_ERROR_NO_DMA_BUFFER;
		rv = -ENOMEM;
		err("%s: usb_buffer_alloc failed %d", dev->dv_xname, rv);
		goto fail0;
	}

	if ((rv = usb_get_descriptor(
	    dev->udev, USB_DT_CONFIG, 0, data, PAGE_SIZE)) < 0) {
		dev->error = TDS_ERROR_NO_DMA_BUFFER;
		err("%s: usb_get_descriptor failed %d", dev->dv_xname, rv);
		goto fail1;
	}
	desc = (struct usb_descriptor_header *)data;

	i = found = 0;
	while (i < rv) {
		if (desc->bDescriptorType == USB_DT_ENDPOINT) {
			epdesc = (struct usb_endpoint_descriptor *)desc;
			if (addr == (epdesc->bEndpointAddress &
			    USB_ENDPOINT_NUMBER_MASK)) {
				found = 1;
				break;
			}
		}
		i += desc->bLength;
		desc = (struct usb_descriptor_header *)((char *)desc +
		    desc->bLength);
	}
	if (found == 0) {
		dev->error = TDS_ERROR_NO_DMA_BUFFER;
		rv = -EINVAL;
		err("%s: endpoint descriptor no found %d", dev->dv_xname, addr);
		goto fail1;
	}

	endpoint->addr = addr;
	endpoint->dir = epdesc->bEndpointAddress & USB_ENDPOINT_DIR_MASK;
	endpoint->xfertype = epdesc->bmAttributes & USB_ENDPOINT_XFERTYPE_MASK;
	endpoint->pktsize = epdesc->wMaxPacketSize;
	usb_buffer_free(dev->udev, PAGE_SIZE, data, dma);

	return (0);

fail1:
	usb_buffer_free(dev->udev, PAGE_SIZE, data, dma);
fail0:
	return (rv);
}

static int
utds_command_dma(utdsep_t endpoint, int len)
{
	utds_t dev = endpoint->dev;
	struct urb *urb = endpoint->urb;
	struct timer_list t;
	int rv = 0;
	unsigned char *buf = endpoint->dmabuf;
	int i;

	if (len & 0x01) {
		dev->error = TDS_ERROR_NOT_ALIGNED;
		return (-EINVAL);
	}
	if (len > endpoint->pktsize) {
		dev->error = TDS_ERROR_NO_DMA_BUFFER;
		return (-EINVAL);
	}

	IFDBGN(2, endpoint->dir != USB_DIR_IN) {
		printk("%s: %s:\n", dev->dv_xname, __FUNCTION__);
		for (i = 0; i < len; i += 2)
			printk("  reg=0x%02x, val=0x%02x\n",
			    buf[i], buf[i + 1]);
	}

	usb_fill_bulk_urb(urb, dev->udev, endpoint->pipe,
	     endpoint->dmabuf, len, utds_command_completion, endpoint);
	init_completion(&endpoint->xdone);
	init_timer(&t);
	t.expires = jiffies + HZ * 5;		/* XXX: approx. */
	t.data = (unsigned long)endpoint;
	t.function = utds_command_timeout;
	add_timer(&t);
	do {
		usb_buffer_map(urb);
		atomic_set(&endpoint->busy, 1);
		if ((rv = usb_submit_urb(urb, GFP_KERNEL)) < 0) {
			del_timer(&t);
			dev->error = TDS_ERROR_DMA_ERROR;
			atomic_set(&endpoint->busy, 0);
			err("%s: usb_submit_urb failed %d", dev->dv_xname, rv);
			goto dma_fail;
		}
		wait_for_completion(&endpoint->xdone);
		if ((rv = urb->status) != 0) {
			if (dev->error == TDS_ERROR_TIMEOUT)
				usb_unlink_urb(endpoint->urb);
			else
				del_timer(&t);
			goto dma_fail;
		}
		usb_buffer_unmap(urb);
	} while (urb->actual_length == 0);
	del_timer(&t);

	IFDBGN(2, endpoint->dir == USB_DIR_IN) {
		printk("%s: %s:\n", dev->dv_xname, __FUNCTION__);
		for (i = 0; i < len; i += 2)
			printk("  reg=0x%02x, val=0x%02x\n",
			    buf[i], buf[i + 1]);
	}

dma_fail:
	usb_buffer_unmap(urb);
	return (rv);
}

static void
utds_command_completion(struct urb *urb, struct pt_regs *regs)
{
	utds_t dev;
	utdsep_t endpoint = (utdsep_t)urb->context;

	dev = endpoint->dev;

	DPRINTF("%s: %s: ep%d, status=%d\n",
	    dev->dv_xname, __FUNCTION__, endpoint->addr, urb->status);

	/* sync/async unlink faults aren't errors */
	if (urb->status &&
	    !(urb->status == -ENOENT || urb->status == -ECONNRESET)) {
		dev->error = TDS_ERROR_DMA_ERROR;
		err("%s: ep%d status %d received",
		    dev->dv_xname, endpoint->addr, urb->status);
	}

	atomic_set(&endpoint->busy, 0);
	complete(&endpoint->xdone);
}

static void
utds_command_timeout(unsigned long data)
{
	utds_t dev;
	utdsep_t endpoint = (utdsep_t)data;

	dev = endpoint->dev;

	dev->error = TDS_ERROR_TIMEOUT;
	printk("%s: command timeout ep%d\n", dev->dv_xname, endpoint->addr);

	complete(&endpoint->xdone);
}

static int
utds_data_dma(utdsep_t endpoint)
{
	utds_t dev = endpoint->dev;
	struct urb *urb = endpoint->urb;
	int rv = 0;

	usb_fill_bulk_urb(urb, dev->udev,
	    endpoint->pipe, dev->iobuf[dev->dma_index].buffer,
	    UTDS_IOBUF_SIZE, utds_data_completion, endpoint);
	atomic_set(&endpoint->busy, 1);
	if ((rv = usb_submit_urb(urb, GFP_ATOMIC)) < 0) {
		dev->error = TDS_ERROR_DMA_ERROR;
		atomic_set(&endpoint->busy, 0);
		clear_bit(UTDS_FLAGS_SAMPLING_START, &dev->flags);
		err("%s: usb_submit_urb failed %d", dev->dv_xname, rv);
	}
	return (rv);
}

static void
utds_data_completion(struct urb *urb, struct pt_regs *regs)
{
	utds_t dev;
	utdsep_t endpoint = (utdsep_t)urb->context;
	int dma_index;

	dev = endpoint->dev;

	DPRINTFN(1, "%s: %s: ep%d, status=%d\n",
	    dev->dv_xname, __FUNCTION__, endpoint->addr, urb->status);

	if (urb->status == 0 && urb->actual_length == UTDS_IOBUF_SIZE) {
		dma_index = (dev->dma_index + 1) % UTDS_IOBUF_NUM;
		if (dma_index == dev->io_index) {
			dev->error = TDS_ERROR_DMA_ERROR;
/*	2006/11/24	H.Ohkubo	*/
/*			dev->error = TDS_ERROR_FIFO_OVERFLOW;	*/
			atomic_set(&endpoint->busy, 0);
			clear_bit(UTDS_FLAGS_SAMPLING_START, &dev->flags);
			err("%s: DMA buffer full\n", dev->dv_xname);
		} else {
			dev->dma_index = dma_index;
			utds_issue_dma(dev);
		}
	} else if (urb->actual_length != UTDS_IOBUF_SIZE) {
		atomic_set(&endpoint->busy, 0);
		clear_bit(UTDS_FLAGS_SAMPLING_START, &dev->flags);

		err("%s: ep%d recieved actual_length %d\n",
		    dev->dv_xname, endpoint->addr, urb->actual_length);
	} else {
		/* sync/async unlink faults aren't errors */
		if (!(urb->status == -ENOENT || urb->status == -ECONNRESET))
			dev->error = TDS_ERROR_DMA_ERROR;
		atomic_set(&endpoint->busy, 0);
		clear_bit(UTDS_FLAGS_SAMPLING_START, &dev->flags);

		err("%s: ep%d status %d received\n",
		    dev->dv_xname, endpoint->addr, urb->status);
	}

	if (waitqueue_active(&dev->waitq))
		wake_up_interruptible(&dev->waitq);
}


/* utds process routines */

static UTDS_INLINE void
utds_init(utds_t dev)
{
	utdsep_t epin, epout;
	int i, n;
	char *buf;

	dev->auxdata.major = UTDSSDH_VERSION_MAJOR;
	dev->auxdata.minor = UTDSSDH_VERSION_MINOR;
	dev->auxdata.size = UTDS_AUXSIZE_DEFAULT;
	memset(dev->auxdata.field, 0, sizeof (dev->auxdata.field));

	dev->dma_index = 0;
	dev->io_index = 0;
	dev->io_offset = 0;
	dev->pktoffset = 0;
	dev->sampling_filter = UTDS_FILTER_DEFAULT;
	dev->sampling_resolutionbit = UTDS_RESOLUTIONBIT_DEFAULT;
	dev->sampling_chmode = UTDS_CHMODE_DEFAULT;
	dev->sampling_fsample = UTDS_FSAMPLE_DEFAULT;
	dev->error = TDS_ERROR_NO_ERROR;

	epin = &dev->endpoints[cmdin];
	epout = &dev->endpoints[cmdout];

	for (i = 0, n = 0; i < 0x40 /* XXXX */; i ++) {
		buf = epout->dmabuf;
		buf[0] = UTDSREG_SPACE;
		buf[1] = 0;
		utds_command_dma(epout, 2);

		utds_command_dma(epin, 2);
		buf = epout->dmabuf;
		if (buf[0] == UTDSREG_SPACE && buf[1] == 0) {
			if (n ++ > 4)
				break;
		} else
			utds_command_dma(epin, 0x40);
	}

#ifdef UTDS_DO_GARBAGE_CLEAR
	dev->initread = 1;
	dev->initcheck = 0;
	dev->refound = 0;
#endif
	utdsreg_reset_memory(dev);

	return;
}

static UTDS_INLINE int
utds_calc_bytes(utds_t dev)
{
	int resolutionbit, chmode, fsample, i;

	resolutionbit = chmode = fsample = 0;

	for (i = 0; resolutionbittbl[i].key != 0; i++)
		if (resolutionbittbl[i].value == dev->sampling_resolutionbit) {
			resolutionbit = resolutionbittbl[i].key;
			break;
		}
	if (resolutionbittbl[i].key == 0)
		return (-EINVAL);

	for (i = 0; chmodetbl[i].key != 0; i++)
		if (chmodetbl[i].value == dev->sampling_chmode) {
			chmode = chmodetbl[i].key;
			break;
		}
	if (chmodetbl[i].key == 0)
		return (-EINVAL);

	for (i = 0; fsampletbl[i].key != 0; i++)
		if (fsampletbl[i].value == dev->sampling_fsample) {
			fsample = fsampletbl[i].key;
			break;
		}
	if (fsampletbl[i].key == 0)
		return (-EINVAL);

	return (resolutionbit * chmode * fsample / 8 /*(bit)*/);
}


static UTDS_INLINE int
utdsio_sampling_start(utds_t dev)
{
	int rv = 0;
	uint8_t status = 0;

	printk("%s: %d.%d.%d\n", dev->dv_xname, TDS_VERSION_MAJOR(utds_ver),
	    TDS_VERSION_MINOR(utds_ver), TDS_VERSION_REVISION(utds_ver));

	if (!test_bit(UTDS_FLAGS_TIME_SET_DONE, &dev->flags)) {
		printk("%s: not synchronized (or set) time yet\n",
		    dev->dv_xname);
		dev->error = TDS_ERROR_NOT_SYNC_TIME;
		rv = -EIO;
	}

	if (rv == 0)
		rv = status = utdsreg_get_status(dev);
	if (rv >= 0) {
		if (!(status & UTDSREG_STAT_REFINPUT)) {
			printk("%s: Ref input not detected\n", dev->dv_xname);
			dev->error = TDS_ERROR_10MHZ_NOT_DETECT;
			rv = -EIO;
		} else {
			if (!test_and_set_bit(
			    UTDS_FLAGS_SAMPLING_START, &dev->flags)) {
				utds_dma_buffer_init(dev);
				dev->totalsize =
				    sizeof (struct tdssdh) +
				    sizeof (uint16_t) + dev->auxdata.size +
				    utds_calc_bytes(dev);

#ifdef UTDS_DO_GARBAGE_CLEAR
				utds_garbage_clear_command(dev);
#endif
				utdsreg_start_sampling(dev);
				utds_issue_dma(dev);
				rv = 0;
			} else {
				dev->error = TDS_ERROR_ALREADY_STARTED;
				rv = -EINVAL;
				err("%s: sampling already started\n",
				    dev->dv_xname);
			}
		}
	}
	return (rv);
}

static UTDS_INLINE int
utdsio_sampling_stop(utds_t dev)
{

	utdsreg_stop_sampling(dev);
	clear_bit(UTDS_FLAGS_SAMPLING_START, &dev->flags);
	return (0);
}

static UTDS_INLINE void
utds_dma_buffer_init(utds_t dev)
{
	utdsep_t endpoint = &dev->endpoints[datain];

	memset(endpoint->dmabuf, 0, endpoint->bufsize);
}

static UTDS_INLINE void
utds_issue_dma(utds_t dev)       
{
	utdsep_t endpoint = &dev->endpoints[datain];

	utds_data_dma(endpoint);
}


/* I/O routines for device node */

static ssize_t
utds_read(struct file *filp, char *buff, size_t count, loff_t *offp)
{
	utds_t dev = (utds_t)filp->private_data;
	struct tdssdh hdr, *hdrp;
	ssize_t readlen = 0;
	unsigned long size;
	int num, resid, index, status = 0, rv, len;
	void *buffer, *buffer2;
	int stchk = 0;

	DPRINTF("%s: %s: count=%d\n", dev->dv_xname, __FUNCTION__, count);

	down(&dev->iosem);
	while (readlen < count && status == 0) {
		if (dev->io_offset == UTDS_IOBUF_SIZE) {
			dev->io_index = (dev->io_index + 1) % UTDS_IOBUF_NUM;
			dev->io_offset = 0;
		}
		num = (dev->dma_index - dev->io_index + UTDS_IOBUF_NUM) %
		    UTDS_IOBUF_NUM;
		resid = num * UTDS_IOBUF_SIZE - dev->io_offset;
		buffer = dev->iobuf[dev->io_index].buffer;

		if (resid == 0 ||
		    (dev->pktoffset == 0 && resid < sizeof (struct tdssdh))) {
			DPRINTFN(1, "%s: %s: event wait\n",
			    dev->dv_xname, __FUNCTION__);

			wait_event_interruptible(dev->waitq,
			    dev->dma_index != dev->io_index ||
			    !test_bit(UTDS_FLAGS_SAMPLING_START, &dev->flags));
			if (!test_bit(UTDS_FLAGS_SAMPLING_START, &dev->flags)) {
				status = -EIO;
				continue;
			}
			if (signal_pending(current)) {
				status = -ERESTARTSYS;
				continue;
			}
			continue;
		}

#ifdef UTDS_DO_GARBAGE_CLEAR
		if (dev->initcheck) {
			while (readlen < count &&
			    dev->io_offset < UTDS_IOBUF_SIZE &&
			    (dev->pktoffset < sizeof (struct tdssdh) ||
			    dev->pktoffset >= sizeof (struct tdssdh) +
			    sizeof (uint16_t) + dev->auxdata.size)) {
				*(buff + readlen) =
				    *(char *)(buffer + dev->io_offset);
				dev->pktoffset ++;
				dev->io_offset ++;
				readlen ++;
				offp ++;

				if (dev->pktoffset == dev->totalsize)
					break;

				if (dev->io_offset + sizeof (struct tdssdh) >
				    UTDS_IOBUF_SIZE) {
					len = UTDS_IOBUF_SIZE - dev->io_offset;
					buffer2 = dev->iobuf[
					    (dev->io_index + 1) %
					    UTDS_IOBUF_NUM].buffer;
					memcpy(&hdr,
					    buffer + dev->io_offset, len);
					memcpy((void *)&hdr + len, buffer2,
					    sizeof (struct tdssdh) - len);
					hdrp = &hdr;
				} else
					hdrp = buffer + dev->io_offset;

				if (IS_SYNC_PATTERN(hdrp)) {
#ifndef SKIP_HEADER_ERROR	/*2007/02/15 T.Kondo*/
					dev->refound = dev->pktoffset;
					dev->pktoffset = 0;
					dev->error = TDS_ERROR_RE_HEADER;
					status = -EFAULT;
					printk("%s: sync-pattern re-found "
					    "offset %d, sec %d\n",
					    dev->dv_xname, dev->refound,
					    hdrp->totalsec0);
					break;
#endif
				}
			}
			if (dev->pktoffset < sizeof (struct tdssdh) ||
			    (dev->pktoffset >= sizeof (struct tdssdh) +
			    sizeof (uint16_t) + dev->auxdata.size &&
			    dev->pktoffset < dev->totalsize))
				continue;
			if (dev->pktoffset == dev->totalsize) {
				dev->pktoffset = 0;
				dev->initcheck = 0;
				continue;
			}
		}
#endif
		if (dev->pktoffset < sizeof (struct tdssdh)) {
			/* check sync-pattern in header */
			if (dev->pktoffset == 0) {
#ifdef UTDS_DO_GARBAGE_CLEAR
				int garbages = 0;

				if (dev->initread == 1) {
					dev->initread = -1;
					printk(
					    "%s: garbage clear", dev->dv_xname);
				}

retry:
#endif
				if (dev->io_offset + sizeof (struct tdssdh) >
				    UTDS_IOBUF_SIZE) {
					len = UTDS_IOBUF_SIZE - dev->io_offset;
					buffer2 = dev->iobuf[
					    (dev->io_index + 1) %
					    UTDS_IOBUF_NUM].buffer;
					memcpy(&hdr,
					    buffer + dev->io_offset, len);
					memcpy((void *)&hdr + len, buffer2,
					    sizeof (struct tdssdh) - len);
					hdrp = &hdr;
				} else
					hdrp = buffer + dev->io_offset;

				if (!(IS_SYNC_PATTERN(hdrp))) {
#ifdef UTDS_DO_GARBAGE_CLEAR
					if (dev->initread == -1) {
						dev->io_offset ++;
						garbages ++;
						resid --;
						if (dev->io_offset ==
						    UTDS_IOBUF_SIZE)
							continue;
						if (resid <
						    sizeof (struct tdssdh))
							continue;
						if (garbages < dev->totalsize)
							goto retry;

						printk(" giveup\n");
					}
#endif
#ifndef SKIP_HEADER_ERROR	/*2006/10/27 H.Ohkubo*/
					dev->error = TDS_ERROR_NO_HEADER;
					status = -EFAULT;
					err("%s: sync-pattern not found\n",
					    dev->dv_xname);

/*	check for HW OVFLW	2007/01/31 H.Ohkubo	*/
					stchk = 0;
					stchk = utdsreg_get_status(dev);
					if ((stchk & UTDSREG_STAT_FIFOOVFLW) != 0) {
						printk("%s: %d FIFO Overflow detected.\n", dev->dv_xname,stchk);
					}

					continue;
#endif	/*SKIP_HEADER_ERROR*/	/*2006/10/27 H.Ohkubo*/

				}

#ifdef UTDS_DO_GARBAGE_CLEAR
				if (dev->initread == -1) {
					dev->initread = 0;
					dev->initcheck = 1;
					printk(" %dbyte\n", garbages);
					continue;
				}
#endif
			}

			if (dev->io_offset + resid > UTDS_IOBUF_SIZE)
				resid = UTDS_IOBUF_SIZE - dev->io_offset;
			size = min(min((int)(count - readlen), resid),
			    (int)(sizeof (struct tdssdh) - dev->pktoffset));
		} else if (dev->pktoffset < sizeof (struct tdssdh) +
		    sizeof (uint16_t) + dev->auxdata.size) {

			/* copy AUX Field */
			size = sizeof (struct tdssdh) +
			    sizeof (uint16_t) + dev->auxdata.size -
			    dev->pktoffset;
			size = min((unsigned long)(count - readlen), size);
			index = dev->pktoffset - sizeof (struct tdssdh);

			DPRINTFN(2, "%s: copy_to_user aux "
			    "index=%d readlen=%d, size=%ld\n",
			    dev->dv_xname, index, readlen, size);
			if ((rv = copy_to_user(buff + readlen,
			    &dev->auxdata + index, size)) == size) {
				dev->error = TDS_ERROR_IO_ERROR;
				status = -EFAULT;
				err("%s: copy_to_user aux failed size=%ld\n",
				    dev->dv_xname, size);
				continue;
			}
			dev->pktoffset += (size - rv);
			readlen += (size - rv);
			offp += (size - rv);

			continue;
		} else {
			if (dev->io_offset + resid > UTDS_IOBUF_SIZE)
				resid = UTDS_IOBUF_SIZE - dev->io_offset;
			size = min((int)(count - readlen), resid);
			if (dev->pktoffset + size > dev->totalsize)
				size = dev->totalsize - dev->pktoffset;
		}

		DPRINTFN(2,
		    "%s: copy_to_user offset=%ld readlen=%d, size=%ld\n",
		    dev->dv_xname, dev->io_offset, readlen, size);
		if ((rv = copy_to_user(buff + readlen,
		    buffer + dev->io_offset, size)) == size) {
			dev->error = TDS_ERROR_IO_ERROR;
			status = -EFAULT;
			err("%s: copy_to_user failed size=%ld\n",
			    dev->dv_xname, size);
			continue;
		}
		dev->pktoffset = (dev->pktoffset + size - rv) % dev->totalsize;
		dev->io_offset += (size - rv);
		readlen += (size - rv);
		offp += (size - rv);

	}


	up(&dev->iosem);

	DPRINTFN(1, "%s: %s: status=%d\n", dev->dv_xname, __FUNCTION__, status);
	return ((status != 0) ? status : readlen);
}

static int
utds_ioctl(
    struct inode *inode, struct file *filp, unsigned int cmd, unsigned long arg)
{
	utds_t dev;
	int rv = 0;

	dev = (utds_t)filp->private_data;

	DPRINTF("%s: %s: start\n", dev->dv_xname, __FUNCTION__);

	switch(cmd) {
	case TDSIO_INIT:
		if (!test_bit(UTDS_FLAGS_SAMPLING_START, &dev->flags))
			utds_init(dev);
		else {
			dev->error = TDS_ERROR_ALREADY_STARTED;
			rv = -EBUSY;
			err("%s: can't initialize while sampling in progress\n",
			    dev->dv_xname);
		}
		break;

	case TDSIO_BUFFER_CLEAR:
		if (!test_bit(UTDS_FLAGS_SAMPLING_START, &dev->flags)) {
			utdsreg_clear_memory(dev);
			DPRINTFN(1, "%s: cleared device FIFO\n", dev->dv_xname);
		} else {
			dev->error = TDS_ERROR_ALREADY_STARTED;
			rv = -EBUSY;
			err("%s: can't clear FIFO while sampling in progress\n",
			    dev->dv_xname);
		}
		break;

	case TDSIO_SYNC_1PPS:
	{
		int count;
		uint8_t status;

		for (count = 0; count < 2; count++) {
			if ((rv = status = utdsreg_get_status(dev)) < 0)
				break;
			rv = 0;
			if (status & UTDSREG_STAT_1PPSINPUT) {
				utdsreg_sync_1pps(dev);
				set_bit(UTDS_FLAGS_PPS_SYNC_DONE, &dev->flags);
				DPRINTFN(1, "%s: synchronized to 1pps\n",
					    dev->dv_xname);
				break;
			}
		}
		if (count >= 2) {
			dev->error = TDS_ERROR_1PPS_NOT_DETECT;
			rv = -EIO;
			err("%s: 1pps input not detected\n", dev->dv_xname);
		}
		break;
	}

	case TDSIO_SET_TIME:
	{
		uint32_t *time = (uint32_t *)arg;

		utdsreg_set_time(dev, *time);
		set_bit(UTDS_FLAGS_TIME_SET_DONE, &dev->flags);
		DPRINTFN(1, "%s: set time 0x%08x\n", dev->dv_xname, *time);
		break;
	}
	case TDSIO_GET_TIME:
		if ((rv = utdsreg_get_time(dev)) >= 0) {
			*(uint32_t *)arg = rv;
			rv = 0;
		}
		break; 

	case TDSIO_GET_YEAR:
		if ((rv = utdsreg_get_year(dev)) >= 0) {
			*(uint32_t *)arg = rv;
			rv = 0;
		}
		break;

	case TDSIO_GET_STATUS:
		if ((rv = utdsreg_get_status(dev)) >= 0) {
			*(int *)arg = rv;
			rv = 0;
		}
		break;

	case TDSIO_SAMPLING_START:
		rv = utdsio_sampling_start(dev);
		break;

	case TDSIO_SAMPLING_STOP:
		rv = utdsio_sampling_stop(dev);
		break;

	case TDSIO_GET_ERROR:
		*(int *)arg = dev->error;
		break;

	case TDSIO_SET_DCOFFSET:
	{
		tdsio_t tdsio = (tdsio_t)arg;

		rv = utdsreg_set_dcoffset(dev, tdsio->key, tdsio->value);
		break;
	}

	case TDSIO_GET_DCOFFSET:
	{
		tdsio_t tdsio = (tdsio_t)arg;

		if ((rv = utdsreg_get_dcoffset(dev, tdsio->key)) >= 0) {
			tdsio->value = rv;
			rv = 0;
		}
		break;
	}

	case TDSIO_BUFFER_RESET:
		if (!test_bit(UTDS_FLAGS_SAMPLING_START, &dev->flags)) {
			utdsreg_reset_memory(dev);
			DPRINTFN(1, "%s: reset device FIFO\n", dev->dv_xname);
		} else {
			dev->error = TDS_ERROR_ALREADY_STARTED;
			rv = -EBUSY;
			err("%s: can't reset FIFO while sampling in progress\n",
			    dev->dv_xname);
		}
		break;

	case TDSIO_GET_1PPSINPUT:
		if ((rv = utdsreg_get_status(dev)) >= 0)
			*(int *)arg = (rv & UTDSREG_STAT_1PPSINPUT) ? 1 : 0;
		break;

	case TDSIO_GET_REFINPUT:
		if ((rv = utdsreg_get_status(dev)) >= 0)
			*(int *)arg = (rv & UTDSREG_STAT_REFINPUT) ? 1 : 0;
		break;

	case TDSIO_GET_10MHZINPUT:
		if ((rv = utdsreg_get_status(dev)) >= 0)
			*(int *)arg =
			    ((rv & UTDSREG_STAT_REFMASK) == UTDSREG_STAT_10MHZ);
		break;

	case TDSIO_SET_CHMODE:
	{
		int chmode = *(uint32_t *)arg;

		if (lookup_value(chmodetbl, chmode) != 0) {
			dev->error = TDS_ERROR_IO_ERROR;
			rv = -EINVAL;
			err("%s: can't set chmode unknown value 0x%x\n",
			    dev->dv_xname, chmode);
		} else if (!test_bit(UTDS_FLAGS_SAMPLING_START, &dev->flags)) {

			dev->sampling_chmode = chmode;
			utdsreg_set_sampling(dev,
			    dev->sampling_filter, dev->sampling_resolutionbit,
			    dev->sampling_chmode, dev->sampling_fsample);
			set_bit(UTDS_FLAGS_SAMPLING_DONE, &dev->flags);
		} else {
			dev->error = TDS_ERROR_ALREADY_STARTED;
			rv = -EBUSY;
			err("%s: can't set chmode while sampling in progress\n",
			    dev->dv_xname);
		}
		break;
	}

	case TDSIO_GET_CHMODE:
		*(uint32_t *)arg = dev->sampling_chmode;
		break;

	case TDSIO_SET_RESOLUTIONBIT:
	{
		int resolutionbit = *(uint32_t *)arg;

		if (lookup_value(resolutionbittbl, resolutionbit) != 0) {
			dev->error = TDS_ERROR_IO_ERROR;
			rv = -EINVAL;
			err("%s: can't set resolutionbit unknown value 0x%x\n",
			    dev->dv_xname, resolutionbit);
		} else if (!test_bit(UTDS_FLAGS_SAMPLING_START, &dev->flags)) {
			dev->sampling_resolutionbit = resolutionbit;
			utdsreg_set_sampling(dev,
			    dev->sampling_filter, dev->sampling_resolutionbit,
			    dev->sampling_chmode, dev->sampling_fsample);
			set_bit(UTDS_FLAGS_SAMPLING_DONE, &dev->flags);
		} else {
			dev->error = TDS_ERROR_ALREADY_STARTED;
			rv = -EBUSY;
			err("%s: can't set resolutionbit "
			    "while sampling in progress\n", dev->dv_xname);
		}
		break;
	}

	case TDSIO_GET_RESOLUTIONBIT:
		*(uint32_t *)arg = dev->sampling_resolutionbit;
		break;

	case TDSIO_SET_FILTER:
	{
		int filter = *(uint32_t *)arg;

		if (lookup_value(filtertbl, filter) != 0) {
			dev->error = TDS_ERROR_IO_ERROR;
			rv = -EINVAL;
			err("%s: can't set filter unknown value 0x%x\n",
			    dev->dv_xname, filter);
		} else if (!test_bit(UTDS_FLAGS_SAMPLING_START, &dev->flags)) {
			dev->sampling_filter = filter;
			utdsreg_set_sampling(dev,
			    dev->sampling_filter, dev->sampling_resolutionbit,
			    dev->sampling_chmode, dev->sampling_fsample);
			set_bit(UTDS_FLAGS_SAMPLING_DONE, &dev->flags);
		} else {
			dev->error = TDS_ERROR_ALREADY_STARTED;
			rv = -EBUSY;
			err("%s: can't set filter while sampling in progress\n",
			    dev->dv_xname);
		}
		break;
	}

	case TDSIO_GET_FILTER:
		*(uint32_t *)arg = dev->sampling_filter;
		break;

	case TDSIO_SET_FSAMPLE:
	{
		int fsample = *(uint32_t *)arg;

		if (lookup_value(fsampletbl, fsample) != 0) {
			dev->error = TDS_ERROR_IO_ERROR;
			rv = -EINVAL;
			err("%s: can't set fsample unknown value 0x%x\n",
			    dev->dv_xname, fsample);
		} else if (!test_bit(UTDS_FLAGS_SAMPLING_START, &dev->flags)) {
			dev->sampling_fsample = fsample;
			utdsreg_set_sampling(dev,
			    dev->sampling_filter, dev->sampling_resolutionbit,
			    dev->sampling_chmode, dev->sampling_fsample);
			set_bit(UTDS_FLAGS_SAMPLING_DONE, &dev->flags);
		} else {
			dev->error = TDS_ERROR_ALREADY_STARTED;
			rv = -EBUSY;
			err("%s: can't set fsample "
			    "while sampling in progress\n", dev->dv_xname);
		}
		break;
	}

	case TDSIO_GET_FSAMPLE:
		*(uint32_t *)arg = dev->sampling_fsample;
		break;

	case TDSIO_SET_YEAR:
	{
		uint32_t *year = (uint32_t *)arg;

		utdsreg_set_year(dev, *year);
		set_bit(UTDS_FLAGS_TIME_SET_DONE, &dev->flags);
		DPRINTFN(1, "%s: set year 0x%08x\n", dev->dv_xname, *year);
		break;
	}

	case TDSIO_SET_AUX:
	{
		tdsauxio_t auxio = (tdsauxio_t)arg;

		if (auxio->auxsize >= 0 && auxio->auxsize < 0x100) {
			dev->auxdata.size = auxio->auxsize;
			memcpy(&dev->auxdata.field,
			    auxio->auxfield, auxio->auxsize);
			DPRINTFN(1, "%s: set AUX Field: size %d\n",
			    dev->dv_xname, auxio->auxsize);
		} else
			rv = -EINVAL;
		break;
	}

	case TDSIO_GET_AUX:
	{
		tdsauxio_t auxio = (tdsauxio_t)arg;

		auxio->auxsize = dev->auxdata.size;
		memcpy(auxio->auxfield, &dev->auxdata.field, dev->auxdata.size);
		break;
	}
		
#ifdef UTDS_DO_GARBAGE_CLEAR
	case TDSIO_GET_REFOUND:
		*(uint32_t *)arg = dev->refound;
		break;
#endif

	case TDSIO_REGIO_READ:
	{
		tdsio_regio_t iocmd = (tdsio_regio_t)arg;
		utdsep_t epin, epout;
		char *buf;

		epin = &dev->endpoints[cmdin];
		epout = &dev->endpoints[cmdout];

		if (iocmd->address > UTDSREG_END) {
			dev->error = TDS_ERROR_IO_ERROR;
			rv = -EINVAL;
			break;
		}

		buf = epout->dmabuf;
		buf[0] = iocmd->address;
		if ((rv = utds_command_dma(epout, 2)) == 0) {
			rv = utds_command_dma(epin, 2);
			buf = epin->dmabuf;
			iocmd->address = buf[0];
			iocmd->value = buf[1];
		}

		break;
	}

#ifdef	TDS_DEBUG
	case TDSIO_REGIO_WRITE:
	{
		tdsio_regio_t iocmd = (tdsio_regio_t)arg;
		utdsep_t endpoint = &dev->endpoints[cmdout];
		char *buf = endpoint->dmabuf;

		if (!(iocmd->address >= UTDSREG_WREG_BEGIN &&
		    iocmd->address <= UTDSREG_END)) {
			dev->error = TDS_ERROR_IO_ERROR;
			rv = -EINVAL;
			break;
		}

		buf[0] = iocmd->address | UTDSREG_WRITE_BIT;
		buf[1] = iocmd->value;
		rv = utds_command_dma(endpoint, 2);

		break;
	}

	case TDSIO_GET_DEBUG:
		*(uint32_t *)arg = utds_debug;
		break;

	case TDSIO_SET_DEBUG:
		utds_debug = *(uint32_t *)arg;
		break;
#endif	/* TDS_DEBUG */

	case TDSIO_GET_VERSION:
		*(uint32_t *)arg = utds_ver;
		break;

	default:
		/* unsupported ioctl */
		return (-ENOTTY);
	}

	DPRINTF("%s: %s: end:\n", dev->dv_xname, __FUNCTION__);
	return (rv);
}

static int
utds_open(struct inode *inode, struct file *filp)
{
	utds_t dev;
	struct usb_interface *iface;
	int minor, rv = 0;

	minor = iminor(inode);

	DPRINTF("%s: %s: minor %d\n", core->name, __FUNCTION__, minor);

	/* prevent disconnects */
	if (down_interruptible(&discsem)) {
		rv = -ERESTARTSYS;
		goto fail0;
	}

	if ((iface = usb_find_interface(&utds_driver, minor)) == NULL) {
		rv = -ENODEV;
		err("%s: usb_find_interface failed %d", core->name, minor);
		goto fail1;
	}
	if ((dev = usb_get_intfdata(iface)) == NULL) {
		rv = -ENODEV;
		goto fail1;
	}
	DPRINTFN(1, "%s: %s\n", dev->dv_xname, __FUNCTION__);

	if (down_interruptible(&dev->iosem)) {
		rv = -ERESTARTSYS;
		goto fail1;
	}
	++dev->open;
	filp->private_data = dev;

	up(&dev->iosem);
	up(&discsem);

	DPRINTF("%s: %s: end\n", dev->dv_xname, __FUNCTION__);
	return (0);

fail1:
	up(&discsem);
fail0:
	DPRINTF("%s: %s: minor %d: failed\n", core->name, __FUNCTION__, minor);
	return (rv);
}

static int
utds_release(struct inode *inode, struct file *filp)
{
	utds_t dev;

	dev = (utds_t)filp->private_data;

	DPRINTF("%s: %s\n", dev->dv_xname, __FUNCTION__);

	if (down_interruptible(&dev->iosem))
		return (-ERESTARTSYS);

	filp->private_data = NULL;
	if (!(--dev->open)) {
		utdsio_sampling_stop(dev);
		if (atomic_read(&dev->endpoints[datain].busy))
			usb_unlink_urb(dev->endpoints[datain].urb);
	}

	up(&dev->iosem);
	return (0);
}


static void
utds_print_info(utds_t dev)
{
	struct usb_device *udev = dev->udev;
	struct usb_device_descriptor *desc = &udev->descriptor;

	printk("%s: 9270 VLBI TimeSync DataSampler at %s\n",
	    dev->dv_xname, udev->bus->bus_name);
	printk("%s: Sampling buffer size %d byte x %d\n",
	    dev->dv_xname, UTDS_IOBUF_SIZE, UTDS_IOBUF_NUM);
	printk("%s:   Var: %2x.%02x\n",
	     dev->dv_xname, desc->bcdUSB >> 8, desc->bcdUSB & 0xff);
	printk("%s:   Class: %xh\n", dev->dv_xname, desc->bDeviceClass);
	printk("%s:   SubClass: %xh\n", dev->dv_xname, desc->bDeviceSubClass);
	printk("%s:   Bus No: %xh\n", dev->dv_xname, udev->bus->busnum);
	printk("%s:   Device No: %xh\n", dev->dv_xname, udev->devnum);
}

static int
utds_probe(struct usb_interface *iface, const struct usb_device_id *id)
{
	struct usb_device *udev = interface_to_usbdev(iface);
	utds_t dev;
	utdsep_t endpoint;
	int i, ep, rv;

	DPRINTF("%s: %s\n", core->name, __FUNCTION__);

	if ((udev->descriptor.idVendor != USB_VENDOR_ID_CYPRESS) ||
	    (udev->descriptor.idProduct != USB_DEVICE_ID_NITSUKI_9270VLBI)) {
		rv = -ENOMEM;
		goto fail0;
	}

	i = 0;		/* XXXX */
/* XXX unit number check */

	/* allocate memory for our device state and initialize it */
	dev = kmalloc(sizeof(struct utds), GFP_KERNEL);
	if (dev == NULL) {
		err("%s%d: kmalloc failed", core->name, i);
		rv = -ENOMEM;
		goto fail0;
	}
	memset(dev, 0, sizeof(struct utds));

	dev->dv_unit = i;
	dev->udev = udev;
	dev->iface = iface;
	snprintf(dev->dv_xname,
	     sizeof (dev->dv_xname), "%s%d", core->name, dev->dv_unit);

	for (ep = 0; ep < maxendpoints; ep++) {
		int epaddrs[] =
		    { UTDS_EP_DATAIN, UTDS_EP_CMDOUT, UTDS_EP_CMDIN };
		int pipetbl[] = {
		    PIPE_CONTROL, PIPE_ISOCHRONOUS,
		    PIPE_BULK, PIPE_INTERRUPT };

		endpoint = &dev->endpoints[ep];
		endpoint->dev = dev;
		utds_get_config(epaddrs[ep], endpoint);
		endpoint->pipe =
		    (pipetbl[endpoint->xfertype] << 30) |
		     __create_pipe(dev->udev, endpoint->addr) | endpoint->dir;

		if ((endpoint->urb = usb_alloc_urb(0, GFP_KERNEL)) == NULL) {
			err("%s: %d: usb_alloc_urb failed", dev->dv_xname, ep);
			rv = -ENOMEM;
			goto allocfail0;
		}

		endpoint->bufsize = endpoint->pktsize;
		if ((endpoint->dmabuf =
		    usb_buffer_alloc(dev->udev,
		    endpoint->bufsize, GFP_KERNEL, &endpoint->dmaaddr)) ==
		    NULL) {
			rv = -ENOMEM;
			err("%s: %d: usb_buffer_alloc failed",
			    dev->dv_xname, ep);
			goto allocfail1;
		}
		atomic_set(&endpoint->busy, 0);
		continue;

allocfail1:
		usb_free_urb(endpoint->urb);
allocfail0:
		goto fail1;
	}

	for (i = 0; i < UTDS_IOBUF_NUM; i++) {
		struct utds_iobuf *iobuf = &dev->iobuf[i];

		if ((iobuf->buffer =
		    usb_buffer_alloc(dev->udev,
		    UTDS_IOBUF_SIZE, GFP_KERNEL, &iobuf->dmaaddr)) == NULL) {
			rv = -ENOMEM;
			err("%s: %d: usb_buffer_alloc failed",
			    dev->dv_xname, ep);
			goto fail2;
		}
	}

	init_MUTEX(&dev->iosem);
	init_waitqueue_head(&dev->waitq);

	/* we can register the device now, as it is ready */
	usb_set_intfdata(iface, dev);
	rv = usb_register_dev(iface, &utds_class);
	if (rv) {
		/* something prevented us from registering this driver */
		err("%s: usb_register_dev failed %d", dev->dv_xname, rv);
		goto fail3;
	}

	utds_init(dev);
	dev->flags = 0;

	utds_print_info(dev);

	return (0);

fail3:
	usb_set_intfdata(iface, NULL);
	i = UTDS_IOBUF_NUM;
fail2:
	for (; i > 0; i--) {
		struct utds_iobuf *iobuf = &dev->iobuf[i];
		usb_buffer_free(dev->udev,
		    UTDS_IOBUF_SIZE, iobuf->buffer, iobuf->dmaaddr);
	}
	ep = maxendpoints;
fail1:
	for (; ep > 0; ep--) {
		endpoint = &dev->endpoints[ep - 1];
		usb_buffer_free(dev->udev,
		    endpoint->bufsize, endpoint->dmabuf, endpoint->dmaaddr);
		usb_free_urb(endpoint->urb);
	}
	kfree(dev);
fail0:
	return (rv);
}


static void
utds_disconnect(struct usb_interface *iface)
{
	utds_t dev;
	utdsep_t endpoint;
	int i;

	DPRINTF("%s: %s\n", core->name, __FUNCTION__);

	/* prevent races with open() */
	down(&discsem);

	dev = usb_get_intfdata(iface);
	usb_set_intfdata(iface, NULL);

	down(&dev->iosem);

	/* give back our minor */
	usb_deregister_dev(iface, &utds_class);

	/* prevent device read, write and ioctl */
	dev->present = 0;
	for (i = 0; i < maxendpoints; i++) {
		endpoint = &dev->endpoints[i];

		/* terminate an ongoing write */
		if (atomic_read(&endpoint->busy)) {
			usb_unlink_urb(endpoint->urb);
			wait_for_completion(&endpoint->xdone);
		}
	}
	up(&dev->iosem);

	/* if the device is opened, release will clean this up */
	if (!dev->open) {
		for (i = 0; i < UTDS_IOBUF_NUM; i++) {
			struct utds_iobuf *iobuf = &dev->iobuf[i];
			usb_buffer_free(dev->udev,
			    UTDS_IOBUF_SIZE, iobuf->buffer, iobuf->dmaaddr);
		}
		for (i = 0; i < maxendpoints; i++) {
			endpoint = &dev->endpoints[i];

			usb_buffer_free(dev->udev, endpoint->bufsize,
			    endpoint->dmabuf, endpoint->dmaaddr);
			usb_free_urb(endpoint->urb);
		}
		kfree(dev);
	}

	up(&discsem);  
}


static int __init
utds_module_init(void)
{
	int rv;

	DPRINTF("%s: %s\n", core->name, __FUNCTION__);

	/* register this driver with the USB subsystem */
	rv = usb_register(&utds_driver);
	if (rv)
		err("%s: usb_register failed %d", core->name, rv);
	return (rv);
}

static void __exit
utds_module_exit(void)
{

	DPRINTF("%s: %s\n", core->name, __FUNCTION__);

	/* deregister this driver with the USB subsystem */
	usb_deregister(&utds_driver);
}
 
module_init(utds_module_init);
module_exit(utds_module_exit);
